#pragma once
#include <vector>
#include <cstring>
#include <stdint.h>
#include <string>

#if (defined(_WIN32) || defined(__WIN32__)) && \
     !defined(WIN32) && !defined(__SYMBIAN32__)
#define WIN32
#endif

#ifdef WIN32

#ifdef MYDLL_EXPORTS
#define DLLCALL __declspec(dllexport)
#else
#define DLLCALL __declspec(dllimport)
#endif // MYDLL_EXPORTS

#endif // WIN32

#define MAX_SCENE_COUNT_PER_CAMERA 4
#define MAX_CHANNEL_COUNT_PER_MICROPHONE 4

namespace NemoSDK
{

	template <typename T>
	class XYArray {
	public:
		XYArray() {
		}

		XYArray(const XYArray& value) {
			copyFrom(value);
		}

		~XYArray() {
			delete[] arrayData;
		}

		void init(int len) {
			if (arrayData)
			{
				delete[] arrayData;
			}

			arrayData = new T[len];
			arrayLen = len;
		}

		T &operator[](int index) {
			return arrayData[index];
		}

		T &operator[](int index) const {
			return arrayData[index];
		}

		XYArray operator=(const XYArray &other) {
			copyFrom(other);
			return *this;
		}

		int length() const {
			return arrayLen;
		}
	protected:
		void copyFrom(const XYArray& other)
		{
			if (&other == this)
			{
				return;
			}

			arrayLen = other.arrayLen;
			delete[] arrayData;
			arrayData = nullptr;
			if (arrayLen == 0)
			{
				return;
			}

			arrayData = new T[arrayLen];

			for (int i = 0; i < arrayLen; i++)
			{
				arrayData[i] = other.arrayData[i];
			}
		}

		T* arrayData = nullptr;
		int arrayLen = 0;
	};

	class XYString : public XYArray<char>
	{
	public:
		XYString()
		{
		}
		XYString(const char* str)
		{
			arrayLen = strlen(str) + 1;
			//delete[] arrayData;
			arrayData = new char[arrayLen];
			memcpy(arrayData, str, arrayLen);
		}

		const char* str() const
		{
			return arrayData;
		}

		XYArray &operator=(const char* other)
		{
			if (arrayData == other)
			{
				return *this;
			}

			arrayLen = strlen(other) + 1;
			delete[] arrayData;
			arrayData = new char[arrayLen];
			memcpy(arrayData, other, arrayLen);
			return *this;
		}
	};

	/* 登录状态 */
	enum SDKLoginState
	{
		SDKLoginState_LoggedIn,    /* 登录完成 */
		SDKLoginState_LoggingIn,   /* 正在登录 */
		SDKLoginState_LoggingOut,  /* 正在登出 */
		SDKLoginState_LoggedOut    /* 登出完成 */
	};

	enum SDKSoftSysType
	{
		SYS_TYPE_CPP_SDK,
		SYS_TYPE_ELECTRON_SDK,
	};

	/* SDK 配置项 */
	struct SDKConfigs
	{
		XYString cloudServer = "";    		/* 云服务地址, 私有云地址非标端口需要带端口 i.e. private.cloud.com:9443, 123.123.123.123:9773 */
		XYString openapiServer = "";  		/* 开放平台地址 */
		XYString openapiToken = "";   		/* 开放平台口令 */
		XYString logFolder = "";      		/* 日志路径 */
		XYString clientId = "";              /* 应用clientId */
		XYString clientSecret = "";          /* 应用clientSecret */
		SDKSoftSysType sysType;         /* 终端类型(会根据终端类型做性能优化) */
		bool enablePushVideo = true; 	/* 使能SDK主动回调裸流给用户 */
		bool enableBulkRoster = false; 	/* 暂未使用 */
		XYString mediaDumpPath;         /* dump调试文件路径 */
	};

	/* 登录后的用户信息 */
	struct SDKLoginInfo
	{
		XYString callUri = "";     		/* 终端标志，小鱼后台给登陆账号分配的唯一呼叫号码，如：123456@DESK， 123456为小鱼系统ID，DESK为终端类型 */
		XYString displayName = "";		/* 用户名称 */
		XYString extUserId = "";   		/* 三方用户ID，同用户登录注册时提供的一样 */
		long userId = 0;				/* 小鱼系统用户ID */
		long deviceId = 0;				/* 软终端无需关注 */
		XYString email = "";             /* 邮箱 */
		XYString peerNumber = "";        /* 用于点对点呼叫号码 */
	};

	/* 会议模式 */
	enum SDKCallMode
	{
		SDKCallMode_AudioVideo,		/* 音视频 */
		SDKCallMode_ContentOnly,	/* 暂不支持 */
		SDKCallMode_HideMode,		/* 暂不支持 */
		SDKCallMode_MonitorMode,	/* 暂不支持 */
		SDKCallMode_AudioOnly,		/* 语音模式 */
	};

	/* 会议信息 */
	struct SDKCallInfo
	{
		int callMode;
		XYString remoteUri;
		XYString remoteName;
		XYString remoteNumber;
		XYString remoteAlias;
		XYString remoteType;
	};

	/* 呼叫状态 */
	enum SDKCallState
	{
		SDKCallState_Connected,
		SDKCallState_Disconnected,
		SDKCallState_Connecting,
		SDKCallState_Disconnecting
	};

	/* 呼叫状态信息 */
	struct SDKCallStateInfo
	{
		SDKCallState callState = SDKCallState_Disconnected;
		XYString sdkError = "";
		XYString disconnectReason = "";
		XYString meetingNumber = "";
		XYString numberType = "";
		XYString displayName = "";
		bool isOnhold = false;
		uint32_t onHoldCount = 0;
		XYString waitingReason = "";
	};

	/* 视频状态 */
	enum SDKVideoState
	{
		SDKVideoState_Observing,
		SDKVideoState_VideoMute,
		SDKVideoState_Requesting,
		SDKVideoState_NoBandwidth,
		SDKVideoState_NoDecoder,
		SDKVideoState_Received,
		SDKVideoState_Telephone,
		SDKVideoState_AudioOnly,
		SDKVideoState_Idle
	};

	/* 呼叫中的远端的视频流信息 */
	struct SDKVideoStreamInfo
	{
		SDKVideoState videoState;
		XYString videoSourceId;
		XYString displayName;
		XYString userNumber;
		XYString extUserId; 			/* 三方用户ID，同用户登录注册时提供的一样 */
		XYString callUri;
		XYString Alias;
		uint32_t videoWidth;
		uint32_t videoHeight;
		uint32_t oriWidth;
		uint32_t oriHeight;
		XYString exDeviceId;
		bool audioMute;
		bool isContent;
		bool isActiveSpeaker;
		uint32_t feccOri;  /* 远端摄像头所具备的能力 */
		void* renderWnd;
		int pid;
	};

	/* 会议参会信息 */
	struct SDKConfInfo
	{
		XYString actSpkUri = "";		/* 当前发言者 */
		XYString chairManUrl = ""; 		/* 主会场 */
		uint32_t participantCount; 		/* 参会人数 */
		uint32_t contentPartCount; 		/* 内容分享数量 */
		uint32_t displayableEpCount;    /* 参会人数除去hide, onhold, contentonly*/
	};

	/* 参会人员类型 */
	enum SDKRosterType
	{
		SDKRosterTypeFull,				/* 全量 */
		SDKRosterTypeIncrement			/* 增量 */
	};

	/* 参会方信息 */
	struct SDKRosterElement
	{
		XYString callUri;				/* 参会者标志, 如：123456@DESK */
		XYString displayName;			/* 显示名称 */
		XYString extUserId;				/* 三方用户ID，同用户登录注册时提供的一样 */
		XYString Alias;					/* 别名 */
		bool isAudioMute;
		bool isVideoMute;
	};

	/* 参会人员 详细信息 */
	struct SDKRosterInfo
	{
		XYArray<SDKRosterElement> addParticipants;    /* 增加的参会方列表， 全量时也包含全部参会方 */
		XYArray<SDKRosterElement> changeParticipants; /* 有状态变化的参会方，增量时使用 */
		XYArray<XYString> delParticipants;            /* 删除的参会方（callUri），增量时使用 */
		SDKRosterType rosterType;                     /* 参会人员类型，全量还是增量 */
		uint32_t totalEpNum;                          /* 总参会人数	*/
	};

	/* 视频状态 */
	enum SDKVideoStatus
	{
		SDKVideoStatus_Normal,
		SDKVideoStatus_LowAsLocalBw,
		SDKVideoStatus_LowAsLocalHardware,
		SDKVideoStatus_LowAsRemote,
		SDKVideoStatus_NetworkError,
		SDKVideoStatus_LocalWifiIssue
	};

	/* 视频帧信息 */
	struct SDKVideoFrame
	{
		uint8_t *buffer;
		size_t bufLen;
		int width;
		int height;
		int sarWidth;
		int sarHeight;
		int rotation;			/* 方向 */
		bool flipping;			/* 镜像 */
	};

	/* 视频分辨率 : 16/9 */
	enum SDKVideoResolution
	{
		VideoResolution90P,			/* 暂不支持 */
		VideoResolution180P,		/* 有效帧率:15fps */
		VideoResolution360P,		/* 有效帧率:15fps|30fps */
		VideoResolution720P,		/* 有效帧率:15fps|30fps */
		VideoResolution1080P,		/* 有效帧率:15fps|30fps|60fps */
		VideoResolution4K,			/* 有效帧率:30fps */
		VideoResolutionUnknown
	};

	/* 视频帧率 */
	enum SDKVideoFrameRate
	{
		VideoFrameRate375,			/* 3.75fps */
		VideoFrameRate500,			/* 5.00fps */
		VideoFrameRate750,			/* 7.50fps */
		VideoFrameRate1500,			/* 15.00fps */
		VideoFrameRate3000,			/* 30.00fps */
		VideoFrameRate6000,			/* 60.00fps */
	};

	/* 视频配置 */
	struct SDKVideoProfile
	{
		SDKVideoResolution maxResPvtx;			/* 最大发送(编码)分辨率 */
		SDKVideoResolution maxResPvrx;			/* 最大接收(解码)分辨率 */
		SDKVideoFrameRate maxFpsPvtx;			/* 最大发送(编码)帧率 */
		SDKVideoFrameRate maxFpsPvrx;			/* 最大接收(解码)帧率 */
	};

	/*
	 *	视频质量级别
	 *	people 720p 时：Low 表示15 fps，Normal和High 表示 30 fps
	 *  people 1080p时：Low 表示15 fps，Normal和High 表示 60 fps，（如果解码能力不够60，会被压低为30的）
	 *
	 *	content 720p 时：Low 表示5 fps， Normal和High 表示 30 fps
	 *  content 1080p时：Low 表示15 fps，Normal和High 表示 30 fps
	 */
	enum SDKVideoQuality
	{
		VideoQualityLow,
		VideoQualityNormal,
		VideoQualityHigh,
		VideoQualityUnknown
	};

	/* 视图请求信息 */
	struct SDKViewReqInfo
	{
		bool isContent;
		XYString callUri = "";            		/* 如果不指定， SDK将按照内部机制自动填充 */
		bool reqFaceDetect = false;             /* set if request face location */
		XYString exDeviceID = "";
		SDKVideoResolution resolution;			/* 请流分辨率 */
		SDKVideoQuality quality;				/* 清流质量 */
	};

	/* 自定义布局请求 */
	struct SDKLayoutReq
	{
		int pageIndex = 0;      			/* 0表示非对称布局，1表示对称第一屏 */
		int maxViewCountPerPage;   			/* 每一屏最大清流数量 */
		bool uiShowLocalWhenPageMode = true;
		XYArray<SDKViewReqInfo> reqList;	/* 清流列表 */
	};

	/* 显示器大小 */
	struct SDKRect
	{
		float x;
		float y;
		float width;
		float height;
	};

	/* 显示器信息 */
	struct SDKMonitorInfo
	{
		XYString monitorName;
		SDKRect minitorRect;
	};

	/* 网络统计信息 */
	struct SDKNetworkStatistics
	{
		int rtt;        /* 往返时间 ms */
		int rxDetectBw; /* 下行带宽 kbps */
		int rxJitter;   /* 下行抖动 ms */
		int rxLost;     /* 下行丢包率 */
		int txDetectBw; /* 上行带宽 kbps */
		int txJitter;   /* 上行抖动 ms */
		int txLost;     /* 上行丢包率 */
	};

	/* 视频统计信息 */
	struct SDKVideoStatistics
	{
		XYString codecType;   /* 编码类型 */
		XYString displayName; /* 显示名称 */
		int actBw;            /* 实际带宽 */
		int frameRate;        /* 帧率 */
		XYString resolution;  /* 分辨率 */
	};

	/* 音频统计信息 */
	struct SDKAudioStatistics
	{
		XYString codecType;   /* 编码类型 */
		XYString displayName; /* 显示名称 */
		int actBw;            /* 实际带宽 */
	};

	/* 音视频会话统计信息 */
	struct SDKAVSessionStatistics
	{
		XYArray<SDKAudioStatistics> audioRxInfo; 	/* 下行音频统计数据 */
		XYArray<SDKAudioStatistics> audioTxInfo; 	/* 上行音频统计数据 */
		XYArray<SDKVideoStatistics> videoRxInfo; 	/* 下行视频统计数据 */
		XYArray<SDKVideoStatistics> videoTxInfo; 	/* 上行视频统计数据 */
	};

	/*呼叫中统计信息 */
	struct SDKStatistics
	{
		SDKAVSessionStatistics people; 			/* 与会人媒体统计信息 */
		SDKAVSessionStatistics content; 		/* 共享内容媒体统计信息 */
		SDKNetworkStatistics network; 			/* 网络状况统计 */
		XYString encrypt;
	};
	
	struct SDKDetailStatistics
	{
		XYString staDBA;
		XYString staETC;
		XYString staARX;
		XYString staATX;
		XYString staVTX;
		XYString staVRX;
		XYString staRoster;
		XYString staCARX;
		XYString staCATX;
		XYString staCVTX;
		XYString staCVRX;
	};
	
	/* 媒体设备信息 */
	struct SDKMediaDevInfo
	{
		XYString devId;
		XYString devName;
		bool isSelected;
	};

	struct SDKConfMgmtInfo
	{
		XYString chirmanUri;
		XYString confMgmtUrl;
		XYString muteSpk;		/* on(mute speaker)|off(unmute speaker)|keep(do nothing) */
		XYString muteMic;		/* mute|unmute|forcemute */
		XYString rename;		/* 当前终端被会控改名(本次会议中有效) */
		bool disableMute;
		bool disableRecord;		/* true: 录制不可用 -> (比如会议室无录制权限, 或者别人已经在录制中了) */
		bool disableContent;	/* 暂未使用 */
		bool disableWhiteboard;
	};

	/* 音视频设备类型 */
	enum SDKMediaDevType
	{
		SDKMediaDevType_Camera,
		SDKMediaDevType_Microphone,
		SDKMediaDevType_Speaker
	};

	/* 远端音频能量信息 */
	struct SDKAudioMeterInfo
	{
		XYString callUri;	/* 参会者标记(123456@DESK) */
		int energy;			/* 声音能量(0-100)*/
		uint32_t pid;
	};

	/* 内容共享状态 */
	enum SDKContentState
	{
		SDKContentState_Idle,
		SDKContentState_Sending,
		SDKContentState_Receiving,
		SDKContentState_NoBandwidth,
		SDKContentState_Error
	};

	struct SDKServerConfig
	{
		XYString confMgmtUrl;
		XYString pstnOutNumbers;
		XYString pcScheduledMeetingUrl;
		XYString serviceLine;
		XYString pcContactUrl;
		XYString pcFolderUrl;
	};

	struct SDKRestApiResult
	{
		XYString code;
		XYString body;
	};

	enum SDKUpdateType
	{
		UpdateType_NAME,
		UpdateType_EMAIL,
		UpdateType_PHOTO
	};

	struct SDKConfCaption
	{
		XYString action;
		XYString content;
		XYString location;
		bool scroll;
		XYString fontSize;
		XYString backgroundAlpha;
		XYString backgroundRGB;
		XYString fontRGB;
		XYString scrollSpeed;
		XYString fontFamily;
	};

	struct SDKInOutInfo
	{
		XYString displayName;
		XYString action; // in or out
		XYString extUserId;
	};

	enum ProxyType
	{
		DIRECT, // none
		HTTP,   // http & wss
		UDP,
		SOCK5,  // SOCKS (V4 or V5) proxy
	};

	struct ProxyNetAddress
	{
		XYString hostAddress = "";
		int port = 0;
		XYString username = "";  // only for SOCKS5 authentication
		XYString password = "";  // only for SOCKS5 authentication
	};

	enum SDKWhiteboardState
	{
		STATE_STARTED,
		STATE_STOPED
	};

	/* 本地录制错误码 */
	enum SDKLocalRecordCode   
	{
		SDKLocalRecordCode_OK,
		SDKLocalRecordCode_NotSupport,
		SDKLocalRecordCode_NoInCall,
		SDKLocalRecordCode_NoSpace,
		SDKLocalRecordCode_AlreadyInRecord,
		SDKLocalRecordCode_Unkonwn
	};

	/* 本地录制类型 */
	enum SDKLocalRecordType 
	{
		SDKLocalRecordType_NonCall,
		SDKLocalRecordType_Call
	};

	/* 本地录制模式 */
	enum SDKLocalRecordMode 
	{
		SDKLocalRecordMode_Film,
		SDKLocalRecordMode_Resources,
		SDKLocalRecordMode_LayoutMux
	};

	struct RecordBusinessInfo
	{
		XYString title;
		XYString speaker;
		XYString confNum;
	};

	struct SDKVideoSource
	{
		/*
		 for local, sourceId means deviceId, if empty string means builtin camara.
					channelId measn sceanId.
		 for remote, sourceId means remote uri,
					channelId is useless
		 */
		XYString sourceId;
		int channelId;
	};

	struct SDKVideoSourceInfo : public SDKVideoSource
	{
		bool bLocalVideoSource;
		bool bContent;
	};

	struct SDKRecordLayoutElement
	{
		SDKVideoSourceInfo sourceInfo;
		XYString displayName = "";
		bool isVideoReceived;
		bool isAudioMute;

		/* 坐标， 均为在canvas 中的比例 */
		float left;
		float top;
		float width;
		float height;
	};

	struct SDKRecordLayout
	{
		
		int canvasAspectWidth; /* 整个画布的比例长宽。  16:9 ,  4:3 等 */
		int canvasAspectHeight;

		XYArray<SDKRecordLayoutElement> elements;
	};

	struct RecordingsFilePath
	{
		XYString localStorePath; //Required
		XYString mobileStorePath = "";
	};

	struct SDKLocalRecording
	{
		int id;                         //database record id
		uint64_t fileSize;
		int sequenceId;
		int needUpload;
		int isLastFile;
		XYString eventId;            //classId or meetingId
		XYString title;
		XYString speaker;
		XYString confNumber;         //meeting number
		XYString recordingId;
		XYString recordingFiles;
		long long timeStamp;
	};

	enum SDKFECCCommand
	{
		FECC_TURN_LEFT,           /* 连续左转 */
		FECC_STEP_LEFT,           /* 向左步进 */
		FECC_TURN_RIGHT,          /* 连续右转 */
		FECC_STEP_RIGHT,          /* 向右步进 */
		FECC_TURN_STOP,           /* 水平转动停止。停止 FECC_TURN_ */
		TILT_CAMERA_TURN_UP,      /* 向上抬起 */
		TILT_CAMERA_STEP_UP,
		TILT_CAMERA_TURN_DOWN,    /* 向下低头 */
		TILT_CAMERA_STEP_DOWN,
		TILT_CAMERA_TURN_STOP,
		FECC_ZOOM_IN,             /* 放大 */
		FECC_ZOOM_OUT,            /* 缩小 */
		FECC_STEP_ZOOM_IN,
		FECC_STEP_ZOOM_OUT,
		FECC_ZOOM_TURN_STOP,
		MOVE_LEFT_TURN,
		MOVE_LEFT_STEP,
		MOVE_RIGHT_TURN,
		MOVE_RIGHT_STEP,
		MOVE_UP_TURN,
		MOVE_UP_STEP,
		MOVE_DOWN_TURN,
		MOVE_DOWN_STEP,
		MOVE_STOP,
		FECC_UNKNOWN
	};

	struct SDKFacePosition
	{
		int32_t faceId;
		uint16_t left;
		uint16_t top;
		uint16_t right;
		uint16_t bottom;
	};

	struct SDKAIParam
	{
		bool isLocal;
		XYString calluri;
		uint32_t participantId;
		uint8_t type;
		uint8_t seq;
		XYArray<SDKFacePosition> positionArr;
	};

	enum SDKFaceType
	{
		SDKFaceType_Detect, // face detect 扫描模式
		SDKFaceType_Tracker, // face tracker
		SDKFaceType_EletronicBadge, // eletronic badge 铭牌模式
	};

	enum SDKHttpMethod
	{
		HTTP_METHOD_GET,
		HTTP_METHOD_POST,
		HTTP_METHOD_PUT,
		HTTP_METHOD_DELETE
	};

	struct SDKHttpRequest
	{
		int what = -1;
		SDKHttpMethod method;
		XYString requestUrl = "";
		XYString requestBody = "";
	};

	struct SDKHttpResponse
	{
		int what;
		XYString response;
	};

	enum SDKVirtualBgMode
	{
		VirtualBgMode_None = 0,     /* 无 */
		VirtualBgMode_BgBlur,   /* 背景虚化 */
		VirtualBgMode_BgImage   /* 虚拟背景 */
	};

	enum SDKBeautyStyle
	{
		BeautyStyle_None = 0,		/* 无 */
		BeautyStyle_Smooth,			/* 磨皮 */
		BeautyStyle_Whiten,			/* 美白 */
		BeautyStyle_Blush			/* 红润 */
	};

	enum SDKFilterStyle
	{
		FilterStyle_None = 0,		/* 无 */
		FilterStyle_Concise,		/* 简洁 */
		FilterStyle_Natural,		/* 自然 */
		FilterStyle_Exquisite,		/* 精致 */
		FilterStyle_Peaceful,		/* 宁静 */
		FilterStyle_Soft,			/* 柔和 */
		FilterStyle_LightLuxury,	/* 轻奢 */
		FilterStyle_Youth,			/* 青春 */
		FilterStyle_Rain,			/* 雨后 */
		FilterStyle_Campus,			/* 校园 */
		FilterStyle_Warmth,			/* 热情 */
		FilterStyle_Refreshing,		/* 清新 */
		FilterStyle_Galaxy,			/* 银河 */
		FilterStyle_Pure,			/* 清澈 */
		FilterStyle_Dawn,			/* 清晨 */
		FilterStyle_Dream,			/* 梦幻 */
		FilterStyle_Highlight,		/* 高光 */
		FilterStyle_Elegant,		/* 典雅 */
		FilterStyle_Nostalgia,		/* 怀旧 */
		FilterStyle_Dark,			/* 暗调 */
		FilterStyle_Retro			/* 老电影 */
	};

	/* 录制状态 */
	enum SDKCloudRecordState
	{
		STATE_PAUSE,
		STATE_RESUME
	};

	/* 录制回调 */
	enum SDKRecordingState
	{
		RECORDING_STATE_IDLE,
		RECORDING_STATE_STARTING,
		RECORDING_STATE_ACTING,
		RECORDING_STATE_STOPING,
		RECORDING_STATE_INACT
	};

	enum SDKExConnectionType
	{
		ConnectionType_UnDefined = 0,
		ConnectionType_TCP = 1,
		ConnectionType_HID = 1 << 1,
		ConnectionType_Extended = 1 << 2
	};

	enum SDKExModuleType
	{
		ModuleType_App = 0,
		ModuleType_Sys,
		ModuleType_Log,
		ModuleType_Unknown,
		ModuleType_Invalid = 0xFF
	};

	struct SDKExConnParam
	{
		SDKExConnectionType connType;
		SDKExModuleType moduleType;
		XYString serverIP;
		uint16_t serverPort;
		size_t maxDataLength;
		uint32_t readInterval;
		XYString name;
		uint32_t keepAliveInterval;
		uint32_t keepAliveTimeout;
	};

	struct SDKExAuthedConnParam : public SDKExConnParam
	{
		XYString deviceID;
		XYString localIP;
		uint32_t authTimeout;
	};

	struct SDKExMasterParam : public SDKExAuthedConnParam
	{
		XYString serial;
	};

	enum SDKExDeviceType
	{
		DeviceType_Master = 0,
		DeviceType_Network,
		DeviceType_Slave,
		DeviceType_IPC,       //tranfer video via IP
		DeviceType_USBCamera, //only transfer vidoe via USB(HID)
		DeviceType_Microphone,
		DeviceType_LocalIPC,  //transfer encoded video via IP, and also transfer raw video via HDMI
		DeviceType_RtspCamera, //trans
		DeviceType_NP30, //trans
		DeviceType_Unknown,
		DeviceType_Invalid = 0xFF
	};

	enum SDKExCapType
	{
		CapType_Undefined = 0,
		CapType_Encoder = 1,
		CapType_Decoder = 1 << 1,
		CapType_Microphone = 1 << 2,
		CapType_EnDecoder = CapType_Encoder | CapType_Decoder,
		CapType_EnMicrophone = CapType_Encoder | CapType_Microphone,
		CapType_DeMicrophone = CapType_Decoder | CapType_Microphone,
		CapType_Both = CapType_Encoder | CapType_Decoder | CapType_Microphone,
	};

	enum SDKExCameraSceneID
	{
		CameraSceneID_Invalid = -1,
		CameraSceneID_Auto = 0,
		CameraSceneID_DistantView,
		CameraSceneID_CloseView,
	};

	enum SDKExDevicePTCap
	{
		ExDevicePT_Undefined = 0,
		ExDevicePT_H264 = 1,
		ExDevicePT_H265 = 1 << 1,
		ExDevicePT_H264_H265 = ExDevicePT_H264 | ExDevicePT_H265,
		EXDevicePT_Invalid = 0xFF
	};

	enum SDKExResolutionIndex
	{
		ResolutionIndex_invalid = -1,
		ResolutionIndex_180P = 0,
		ResolutionIndex_360P,
		ResolutionIndex_720P,
		ResolutionIndex_1080P,
		ResolutionIndex_4K,
		ResolutionIndex_Count
	};

	struct SDKExVideoCap
	{
		SDKExVideoCap() :
			instanceLimit(0)
		{
			for (int i = 0; i < ResolutionIndex_Count; i++)
			{
				expendMap[i] = 0;
			}
		}

		SDKExVideoCap &operator=(const SDKExVideoCap &other)
		{
			instanceLimit = other.instanceLimit;
			for (int i = 0; i < ResolutionIndex_Count; i++)
			{
				expendMap[i] = other.expendMap[i];
			}
			return *this;
		};

		unsigned int instanceLimit;
		float expendMap[ResolutionIndex_Count];

	};

	struct SDKExSceneEncoderCap : public SDKExVideoCap
	{
		SDKExSceneEncoderCap() :
			SDKExVideoCap(),
			sceneId(CameraSceneID_Invalid),
			codecType(ExDevicePT_H264),
			maxFrameRate(30.0)
		{
		}

		SDKExSceneEncoderCap &operator=(const SDKExSceneEncoderCap &other)
		{
			SDKExVideoCap::operator=(other);
			sceneId = other.sceneId;
			codecType = other.codecType;
			maxFrameRate = other.maxFrameRate;
			return *this;
		};

		SDKExCameraSceneID sceneId;
		SDKExDevicePTCap codecType;
		float maxFrameRate;
	};

	struct SDKExEncoderCap
	{
		SDKExEncoderCap() :
			sceneNum(0)
		{
		}

		SDKExEncoderCap &operator=(const SDKExEncoderCap &other)
		{
			sceneNum = other.sceneNum;
			for (int i = 0; i<sceneNum; i++)
			{
				sceneEncCaps[i] = other.sceneEncCaps[i];
			}

			return *this;
		}

		int sceneNum;
		SDKExSceneEncoderCap sceneEncCaps[MAX_SCENE_COUNT_PER_CAMERA];
	};

	struct SDKExMicChannel
	{
		SDKExMicChannel() :channelID(0) {}
		SDKExMicChannel(const SDKExMicChannel& other) :channelID(other.channelID) {}
		SDKExMicChannel &operator=(const SDKExMicChannel &other)
		{
			channelID = other.channelID;
			return *this;
		};

		uint32_t channelID;
	};

	struct SDKExMicCap
	{
		SDKExMicCap() :channelNums(0) {}
		SDKExMicCap(const SDKExMicCap& other) :
			channelNums(other.channelNums)
		{
			for (int i = 0; i < channelNums; i++)
			{
				channelArr[i] = other.channelArr[i];
			}
		}

		SDKExMicCap &operator=(const SDKExMicCap &other)
		{
			channelNums = other.channelNums;
			for (int i = 0; i < channelNums; i++)
			{
				channelArr[i] = other.channelArr[i];
			}
			return *this;
		};

		int  channelNums;
		SDKExMicChannel channelArr[MAX_CHANNEL_COUNT_PER_MICROPHONE];
	};

	struct SDKExDeviceInfo
	{
	public:
		SDKExDeviceInfo() :
			isLocal(false),
			deviceID(""),
			name(""),
			ip(""),
			type(DeviceType_Unknown),
			capType(CapType_Undefined),
			version(""),
			model(""),
			productId(""),
			hwVersion(""),
			connType(ConnectionType_UnDefined)
		{};

		SDKExDeviceInfo(const SDKExDeviceInfo& other)
		{
			isLocal = other.isLocal;
			deviceID = other.deviceID;
			name = other.name;
			ip = other.ip;
			type = other.type;
			capType = other.capType;
			version = other.version;
			model = other.model;
			productId = other.productId;
			hwVersion = other.hwVersion;
			connType = other.connType;
		};

		SDKExDeviceInfo &operator=(const SDKExDeviceInfo &other)
		{
			isLocal = other.isLocal;
			deviceID = other.deviceID;
			name = other.name;
			ip = other.ip;
			type = other.type;
			capType = other.capType;
			version = other.version;
			model = other.model;
			productId = other.productId;
			hwVersion = other.hwVersion;
			connType = other.connType;

			return (*this);
		};

		bool isLocal;
		std::string deviceID;
		std::string name;
		std::string ip;
		SDKExDeviceType type;
		SDKExCapType capType;
		std::string version;
		std::string model;
		std::string productId;
		std::string hwVersion;
		SDKExConnectionType connType;
	};

	struct SDKExCapInfo
	{
		SDKExCapInfo() :encCap(), decCap(), micCap(), extraExpend(0.0) {}

		SDKExCapInfo(const SDKExCapInfo& other) :
			encCap(other.encCap),
			decCap(other.decCap),
			micCap(other.micCap),
			extraExpend(other.extraExpend) {}

		SDKExCapInfo &operator=(const SDKExCapInfo &other)
		{
			encCap = other.encCap;
			decCap = other.decCap;
			micCap = other.micCap;
			extraExpend = other.extraExpend;

			return *this;
		}
		SDKExEncoderCap encCap;
		SDKExVideoCap decCap;
		SDKExMicCap micCap;
		float extraExpend;
	};

	struct SDKExCapDeviceInfo : public SDKExDeviceInfo, public SDKExCapInfo
	{
	public:
		SDKExCapDeviceInfo() :SDKExDeviceInfo(), SDKExCapInfo() {}

		SDKExCapDeviceInfo(const SDKExCapDeviceInfo& other) : SDKExDeviceInfo(other), SDKExCapInfo(other) {}

		SDKExCapDeviceInfo &operator=(const SDKExCapDeviceInfo &other)
		{
			SDKExDeviceInfo::operator=(other);
			SDKExCapInfo::operator=(other);

			return *this;
		}
	};

	struct SDKConfHost
	{
		XYString meetingId;
		bool isHost;
	};

	enum SDKInteractiveToolEventType {
		SDKInteractiveToolEventType_Start,
		SDKInteractiveToolEventType_Stop,
		SDKInteractiveToolEventType_Publish,
		SDKInteractiveToolEventType_ResultClose
	};

	enum SDKInteractiveToolProcessType {
		SDKInteractiveToolProcessType_None,
		SDKInteractiveToolProcessType_SignIn,
		SDKInteractiveToolProcessType_Answer,
		SDKInteractiveToolProcessType_Vote,
		SDKInteractiveToolProcessType_Evaluation
	};

	struct SDKInteractiveCopywriting
	{
		// 左边提示卡片文案
		XYString notifyContent{ "" };
		XYString notifyLabel{ "" };
		// 弹窗文案
		XYString dialogTitle{ "" };
		XYString dialogContent{ "" };
		XYString dialogSubContent{ "" };
		XYString dialogLabel{ "" };
	};

	struct SDKInteractiveToolInfo
	{
		SDKInteractiveToolEventType eventType;		// 互动工具事件
		SDKInteractiveToolProcessType processType;	// 互动工具类型
		bool endAuto{ false };						// 是否自动结束
		int duration{ 0 };							// 时长
		long long endUtcTime{ 0 };					// 结束时间
		XYString webViewUrl;						// webViewUrl
		XYString questionnaireId;
		XYString meetingId;
		XYString business;
		SDKInteractiveCopywriting copywriting;		// 参考文案
	};

	struct SDKSubmitSignResult {
		int code;
		XYString status;
	};

	struct SDKRegion {
		int x = 0;		// the X-coordinate of the top-left point of region
		int y = 0;		// the Y-coordinate of the top-left point of region
		int width = 0;  // width of region
		int height = 0; // height of region
	};

	// It used to startContentCapture function as a param.
	struct SDKContentInfo {
		int64_t     viewId = 0;		// not used
		bool        withAudio = false;	// content audio
		SDKRegion   region;		// for region share, fullscreen share not need this
	};

	class INemoSDKCallbacks
	{
	public:
		/* 返回登录状态 */
		virtual void onLoginStateChanged(SDKLoginState state, const XYString& code, const SDKLoginInfo& info) = 0;

		/*
		* 用户被登出回调，需要重新登录
		* 1） 4000 ：多个重复长连接建立
		* 2） 4001 ：用户在另一台设备登录
		* 3） 4003 ：无效的SK、登录过期(默认1年)
		*/
		virtual void onKickOut(const XYString& code) = 0;

		/* 返回呼叫状态 */
		virtual void onCallStateChanged(const SDKCallStateInfo &callStateInfo) = 0;

		/* 收到点对点(p2p)呼叫 */
		virtual void onCallInvited(const SDKCallInfo &info) = 0;

		/* 是否检测到啸叫(用户可降低声音或关闭麦克风结束啸叫) */
		virtual void onHowlingDetected(bool detected) = 0;

		/*
		* 在已登录的情况下这个回调接口上报网络连接状况，未登录的情况下不上报
		* 1) 如果是在呼叫中:
		*	  如果App层收到 onNetworkStateChanged( connected==false), App层应该禁止用户除了挂断外的其他操作。
		*	  如果在 40 秒内恢复网络连接, 通话继续进行，否则App层会收到 OnCallStateChanged(state==disconnected) 消息
		* 2) 如果不是在呼叫中:
		*	  如果App层收到 onNetworkStateChanged( connected==false), App层应该禁止用户发起呼叫操作。
		*/
		virtual void onNetworkStateChanged(bool connected) = 0;

		/* 会议信息变化上报 */
		virtual void onConfInfoChanged(const SDKConfInfo &info) = 0;

		/**
		 * 处理会控消息
		 * 控制操作：静音、非静音
		 * 控制状态：举手发言、取消举手、结束发言
		 *
		 * @param SDKConfMgmtInfo#muteMic:      mute/unmute
		 * @param SDKConfMgmtInfo#disableMute:  是否为强制静音 true强制静音
		 */
		virtual void onConfMgmtChanged(const SDKConfMgmtInfo & info) = 0;

		/* 参会信息变化上报 */
		virtual void onRosterChanged(const SDKRosterInfo &info) = 0;

		/* 返回内容共享状态 */
		virtual void onContentStateChanged(SDKContentState state) = 0;

		/* 返回远端视频流状态信息 */
		virtual void onVideoStreamChanged(const XYArray<SDKVideoStreamInfo>& streamArray) = 0;

		/*
		* 上报视频流裸数据
		* dataSourceId ： SDK为没路视频流的分配的资源标志
		* 本地预览的sourceId 固定为：LocalPreviewID
		* 本地预览的固定帧率为30fps@720p，推流最大帧率也为30fps
		*/
		virtual void onVideoFrameReady(const char* dataSourceId, const SDKVideoFrame& videoFrame) = 0;

		/* 某种因素引起视频状态的变化 */
		virtual void onVideoStatusChanged(SDKVideoStatus status) = 0;

		/*
		* 本地网络状况上报
		* 回调频率根据入会终端期望带宽和DBA(动态带宽调整)探测带宽的比例计算
		* 计算的Level不同会上报，同时也跟终端网络的稳定性有关
		* 一共4档：1、2、3、4 分别为：差、中、良、优
		*/
		virtual void onNetworkIndicatorLevel(int level) = 0;

		/* 本地设备发生变化上报 */
		virtual void onDeviceChanged(const SDKMediaDevType& devType) = 0;

		/* 上报麦克风状态 */
		virtual void onMicMuted(bool muted) = 0;

		/* Mic能量改变(0-100) */
		virtual void onMicEnergyReported(int energy) = 0;

		/* SDK错误回调 */
		virtual void onError(const XYString& code) = 0;

		/* 音频旁路 */
		virtual void onMicDataByAEReady(const void* data, int length, int formatType, int samplesPerSec, int numChannels, int bitsPerSample) {};

		/* 会控字幕 */
		virtual void onConfCaptionChanged(const SDKConfCaption& notify) {};

		/* 出入会通知 */
		virtual void onInOutReminder(const XYArray<SDKInOutInfo>& info) {};

		/* 会控清空举手信息回调 */
		virtual void onConfHandupCancelled() {};
		
		/* 发言者列表 */
		virtual void onSpeakersChanged(const XYArray<SDKAudioMeterInfo> &meters) {};

		/* 人脸识别数据回调 */
		virtual void onAIFaceRecv(const SDKAIParam& info) {};

		/* @link: getFaceInfo回调 */
		virtual void onAIFaceInfo(const XYString& data) {};

		/* @link: getGPUInfo回调 */
		virtual void onGPUInfoResult(const XYArray<XYString> &gInfo) {};

		/**
		 * 云端录制状态通知
		 * isStart: 开始/结束
		 * uri: 录制发起者名称
		 */
		virtual void onRecordStatusNotification(int callIndex, bool isStart, const XYString& uri, const XYString& callUri, const XYString& status) = 0;

		/**
		 * 云端录制回调
		 * recordState: 录制状态
		 * reason: 原因, 见(json error code)
		 */
		virtual void onRecordingStateChanged(SDKRecordingState recordState, const XYString& reason,  bool isRecorded) = 0;

		/* 会控关闭摄像头回调 */
		virtual void onMeetingMuteQuery(const XYString& meetingId, const XYString& key) {};

		/* 日志上传回调 */
		virtual void onLoguploadResult(const SDKRestApiResult& result) {};

		virtual void onExDevicesChanged() {};

		/* 会控主持人回调 */
		virtual void onConfHostChanged(const SDKConfHost& meetingHost) {};

		/* 互动工具回调 */
		virtual void onInteractiveToolInfoResult(const SDKInteractiveToolInfo& result) {};

		/* 签到回调 */
		virtual void onSubmitSignatureInfosResult(const SDKSubmitSignResult& result) {};
	};

	class INemoSDK
	{
	public:
		/*
		* 初始化并启动SDK
		*/
		virtual bool startup(INemoSDKCallbacks *cb, const SDKConfigs& sdkConfigs) = 0;

		/* 停止SDK，关闭应用前调用该方法 */
		virtual void shutdown() = 0;

		/* 使用该企业下 XYLINK 账号登录(小鱼账号必须跟sdk开通时的所属企业相同, 否则会鉴权失败) */
		virtual void loginXYLinkAccount(const char* userName, const char* password) = 0;

		/* 使用验证码登录 */
		virtual void loginXYLinkByVerifyCode(const char* phoneNum, const char* identificationNum) = 0;
		/*
		* 使用第三方账户登录
		* extID为企业标志，联系小鱼销售人员获取
		* extUserId为三方用户ID，同一企业不能相同
		* displayName 参数不能包含以下字符
		* ';' ':' ',' '=' '-' '(' ')' '[' ']' '{' '}' '@' '#' '*' '\\' '\r' '\n' '\t'
		*/
		virtual void loginExternalAccount(const char* extID, const char* extUserId, const char* displayName) = 0;

		/* for compatible, The next version will be deprecated */
		virtual void loginExternalAccountOld(const char* extID, const char* extUserId, const char* displayName) = 0;

		/* 企业账号登录 */
		virtual void loginEnterpriseAccount(const char* extID, const char* account) = 0;

		/* 登出 */
		virtual void logout() = 0;

		/*
		* 发起呼叫
		* 如果 displayName 为空的话，就使用 loginExternalAccount 时传入的displayName。
		* 如果不为空的话，就使用这个新的。
		*
		* isVideoMute: 关闭视频入会
		* isAudioMute: 关闭音频入会
		* meetingID: 预约会议id
		*/
		virtual void makeCall(const char* number, const char* meetingPwd, SDKCallMode mode, const char* displayName = "", 
			bool isVideoMute = false, bool isAudioMute = false, bool isMonitor = false, const char* meetingID = "") = 0;

		/* 挂断呼叫 */
		virtual void hangup() = 0;

		/* 接受点对点(p2p)被叫 */
		virtual void acceptCall() = 0;

		/* 拒绝点对点(p2p)被叫*/
		virtual void rejectCall() = 0;

		/* 会议中停止/开始发送摄像头视频，并且通知远端 */
		virtual void setVideoMute(bool bMute) = 0;

		/* 会议中停止/开始发送Mic音频，并通知远端 */
		virtual void setMicMute(bool bMute) = 0;

		/* 设置本地预览视频分辨率 */
		virtual void setLocalPreviewResolution(SDKVideoResolution res) = 0;

		/* 
		 * 获取视频裸数据(SDK会主动推流@link#onVideoFrameReady, 如果需要主动拉流
		 * 设置@startup#SDKConfigs.enablePushVideo=false)
		 */
		virtual bool getVideoFrame(const char* sourceId, int sourceKey, SDKVideoFrame& frame) = 0;

		/*
		 * startContentSharing: 开启内容共享
		 * win 默认全屏
		 * linux 默认起始坐标(0, 0), 1920 x 1080
		 *  @deprecated use @link# startContentSharing(SDKContentInfo contentInfo)
		 */
		virtual void startContentSharing(bool withDesktopAudio) = 0;

		/*
		 * startContentSharing: 开启内容共享 - linux only, win not support
		 * 区域: 起始位置(startX, startY), rectWidth x rectHeight
		 * @deprecated use @link# startContentSharing(SDKContentInfo contentInfo)
		 */
		virtual void startContentSharing(bool withDesktopAudio, int startX, int startY, int rectWidth, int rectHeight) = 0;

		/*
		 * 屏幕共享
		 */
		virtual void startContentSharing(const char* monitor, const SDKContentInfo& contentInfo) = 0;

		/// @brief 更新共享内容采集的区域
		/// @param monitor 显示器名称
		/// @param x 采集区域的x坐标
		/// @param y 采集区域的y坐标
		/// @param width 采集区域的宽度
		/// @param height 采集区域的高度
		virtual void updateContentCaptureRegion(int x, int y, int width, int height) = 0;

		/* 暂停内容共享 */
		virtual void suspendContentShare() = 0;

		/* 继续内容共享 */
		virtual void restartContentShare() = 0;

		/* 停止内容共享 */
		virtual void stopContentSharing() = 0;

		/* 获取通话统计信息 */
		virtual const SDKStatistics& getStatistics() = 0;

		/* 获取详细统计信息*/
		virtual const SDKDetailStatistics& getDetailStatistics() = 0;
		
		/*
		* 启动摄像头采集 & 用于摄像头检测
		* 会中或者会议建立以后（onCallStateChanged(connected)）
		*/
		virtual void startCamera() = 0;

		/*
		* 关闭摄像头采集 & 用于摄像头检测
		* 会中或者退会之前关闭（onCallStateChanged(disconnected)）
		*/
		virtual void stopCamera() = 0;

		/*
		* 订阅参会者列表(最大20个)，优先请流
		* uri为参会者设备唯一标志，如：123456@DESK，123456为小鱼用户ID，DESK(桌面)为终端类型
		*/
		virtual void subscribeRoster(const XYArray<XYString>& uriList) = 0;

		/* 获取显示器列表 */
		virtual const XYArray<SDKMonitorInfo>& getMonitorList() = 0;

		/* 获取设备列表 */
		virtual const XYArray<SDKMediaDevInfo>& getDevList(SDKMediaDevType t) = 0;

		/* 通用选择设备 (Linux平台通话中选择设备, 调用chooseDev(SDKMediaDevType, const char*, bool)接口, 通话外两者都行) */
		virtual void chooseDev(SDKMediaDevType t, const char* devId) = 0;

		/* Linux平台选择设备, isInConf: 是否在会中, 用于会中切换设备 */
		virtual void chooseDev(SDKMediaDevType t, const char* devId, bool isInConf) = 0;

		/* 按需清流 */
		virtual void requestLayout(const SDKLayoutReq& layoutReq) = 0;

		/* 设置视频配置，呼叫前设置有效 */
		virtual void setVideoProfile(const SDKVideoProfile& videoProfile) = 0;

		/* 开启远端音频能量峰值计量 */
		virtual void setAudioPeakMeterEnabled(bool enable) = 0;

		/* 获取远端音频能量 @deprecated Use {@link #onSpeakersChanged(SDKAudioMeterInfos)} */
		virtual const XYArray<SDKAudioMeterInfo>& getAudioPeakMeters() = 0;

		/* 获取本地麦克分能量波动 */
		virtual int getMicPeakMeter() = 0;

		/* 获取本地扬声器能量波动 */
		virtual int getSpeakerPeakMeter() = 0;

		/* 获取扬声器当前音量 */
		virtual int getSpeakerVolume() = 0;

		/* 设置本地扬声器能量 */
		virtual void setSpeakerVolume(int volume) = 0;

		/* 设置本地麦克风音量，范围[0, 100] */
		virtual void setMicVolume(int value) = 0;

		/* 获取本地麦克风音量范围值，范围[0, 100] */
		virtual int getMicVolume() = 0;

		/* 开始播放测试文件用于扬声器检测 */
		virtual void startPlayWavFile(const char* wavfile, int loopTimes) = 0;

		/* 结束播放测试文件用于扬声器检测 */
		virtual void stopPlayWavfile() = 0;

		/* 使能麦克风检测 */
		virtual void enableMicTest(bool enable) = 0;

		/* 举手发言 */
		virtual void sendSpeakingRequest() = 0;

		/* 发言结束 */
		virtual void sendSpeakingEnd() = 0;

		/* 取消举手 */
		virtual void cancelSpeakingRequest() = 0;

		/* 音频旁路 */
		virtual void setMicDataByAEEnabled(bool enabled) = 0;

		/* 会控url */
		virtual const XYString& getConfMgmtUrl(const char* confNum) = 0;

		/**
		 * 遥控远端摄像头: 远端视频流信息(对应UI上的视频窗格)SDKVideoStreamInfo包含所需参数
		 *
		 * 当前摄像头所支持的指令集, @link onVideoStreamChanged#SDKVideoStreamInfo.feccOri
		 * (feccOri & 1 << 1) != 0 : 支持水平方向上的转动 (左右)
		 * (feccOri & 1 << 2) != 0 : 支持垂直方向上的转动 (上下)
		 * (feccOri & 1 << 4) != 0 : 支持缩放
		 * 
		 * @param callUri: 远端callUri
		 * @param SDKFECCCommand: 指令
		 * @param angle: 角度(0~360)
		 */
		virtual void farEndHardwareControl(const char* callUri, SDKFECCCommand command, int angle) = 0;

		/* ae audio dump, 入会前设置, dump本次呼叫中的音频数据供分析噪声,回声使用 */
		virtual void enableAudioDump() = 0;

		virtual void setMediaDumpMask(uint32_t dumpMask) = 0;

		/* 是否开启小鱼回声消除功能(默认启用) */
		virtual void enableAECMode(bool enable) = 0;

		/* 设置mic增益, 通过调节mic采集增益来放大输出声音, 但是噪声也会放大 [0 - 20] */
		virtual void setAudioAutoGainCtrl(int gain) = 0;

		/* 
		 * 广播本地电子铭牌到远端: 开启后接收端可以收到广播出去的人脸信息
		 * 例如: 终端A开启了广播本地电子铭牌, B终端请求了A的画面, 且A在B上的布局画面符合显示条件(例:A在大画面), 那么终端B就可以去显示A的铭牌信息
		 */
		virtual void broadcastEletronicBadge(bool enable) = 0;

		/* 
		 * 人脸识别模式
		 * 1. 会议中人脸识别
		 *     会议中, 显示人脸识别, 本地上大屏时候调用 enableFaceDetectMode(SDKFaceType_Detect, true)
		 *     会议中, 显示人脸识别且本地不在大屏时候,或者不显示人脸识别, 调用 enableFaceDetectMode(SDKFaceType_Detect, false) 
		 * 2. 电子铭牌
		 *     会议中, 本终端电子铭牌被激活, 本地上大屏时候调用 enableFaceDetectMode(SDKFaceType_EletronicBadge, true)
		 *     会议中, 本终端电子铭牌被激活且本地不在大屏时候, 或者电子铭牌被禁用, 调用 enableFaceDetectMode(SDKFaceType_EletronicBadge, false)
		 */
		virtual void enableFaceDetectMode(SDKFaceType faceType, bool enable) = 0;

		/* 获取人脸信息详情 */
		virtual void getFaceInfo(const XYArray<int32_t>& faceId) = 0;

		/* 虚拟背景模式 */
		virtual void setVirtualBgMode(SDKVirtualBgMode mode) = 0;

		/* 虚拟背景图片, 仅支持jpg格式 (其他格式得话应用层自己处理格式转换跟存储逻辑, 可参考demo) */
		virtual void setVirtualBgImage(const char* imgPath) = 0;

		/* 设置美颜效果 */
		virtual void setVideoBeautyEffect(SDKBeautyStyle style, int effectLevel) = 0;

		/* 设置滤镜效果 */
		virtual void setVideoFilterEffect(SDKFilterStyle style, int effectLevel) = 0;

		/* 获取gpu信息 */
		virtual void getGPUInfo() = 0;

		/* 开启gpu加速, 如果启用虚拟背景跟美颜, 在初始化之后可以开启gpu加速 */
		virtual void setEnableGPUs(const XYArray<XYString> &enableGPUs) = 0;

		/* 日志上传 */
		virtual void logUpload(const char* comments) = 0;

		/* 开启云端录制 */
		virtual void startCloudRecord() = 0;

		/* 停止云端录制 */
		virtual void stopCloudRecord() = 0;

		/* 暂停云端录制 现有客户端暂无实现 */
		virtual void pauseCloudRecord(SDKCloudRecordState pauseOrResume) = 0;

		/* 
		 * 切换通话模式 
		 * @param: audioMode: ture: 语音模式, false:	音视频模式
		 * audioMute: 当前mic状态
		 * videoMute: 当前camera状态
		 */
		virtual void switchCallMode(bool audioMode, bool audioMute, bool videoMute) = 0;

		virtual void startMaster(const SDKExMasterParam& param) = 0;

		virtual void stopMaster() = 0;

		virtual std::vector<SDKExCapDeviceInfo> getExDeviceList() = 0;

		/* 会中邀请h323设备, remoteUri: 终端calluri,  key: 号码 */
		virtual void sendDtmf(const char *remoteUri, const char *key) = 0;

		/// @brief 释放音频焦点, 会外设备检测完成之后, 如果需要释放音频的话调用
		virtual void releaseAudioFocus() = 0;

		/// @brief 签到
		/// @param url 签到url 从会控发起的ws消息里面拿到
		/// @param identifier 签到id 从会控发起签到的ws消息里面拿到
		virtual void signIn(const char* url, const char* identifier) = 0;

        /**
         * 发送采集到的桌面数据
         * @param argb argb
         * @param bufLen bufLen
         * @param width width
         * @param height height
         * @param stride stride
         */
        virtual void putContentData(void *argb, size_t bufLen, int width, int height, int stride) = 0;

        /**
         * 音频设备控制辅助
         * @param enable 切换设备或者会外检测时调用
         * @param wavfile wav文件
         */
		virtual void enablePulseHelper(bool enable) = 0;

        virtual void convertYUV2ARGB(const uint8_t* data, uint8_t* dst_argb, size_t bufLen, int width, int height, int rotate, bool flip) = 0;
	};
}

extern "C"
{

#ifdef WIN32
	DLLCALL NemoSDK::INemoSDK* _cdecl getNemoSdkInstance();
#endif // WIN32

#ifdef LINUX
	NemoSDK::INemoSDK* getNemoSdkInstance();
#endif // LINUX

};

typedef NemoSDK::INemoSDK* (*GET_NEMOSDK_INSTANCE)();
